/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.4 Funktionen Fuzzy
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.fuzzylib.fuzzylib;

import de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogischerWert;

/**
 * Ein Term und seine Zugeh&ouml;rigkeit.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
public class Term extends LogischerWert {

	/**
	 * Die Fuzzy-Variable zu der dieser Term geh&ouml;rt.
	 */
	private final FuzzyVariable fuzzyVariable;

	/**
	 * Der Name des Terms.
	 */
	private final String name;

	/**
	 * Konstruiert einen Term mit der Zugeh&ouml;rigkeit 0.0.
	 *
	 * @param fuzzyVariable
	 *            Die Fuzzy-Variable zu der der Term geh&ouml;rt
	 * @param name
	 *            Der Name des Terms
	 */
	Term(final FuzzyVariable fuzzyVariable, final String name) {
		this(fuzzyVariable, name, 0.0f);
	}

	/**
	 * Konstruiert einen Term.
	 *
	 * @param fuzzyVariable
	 *            Die Fuzzy-Variable zu der der Term geh&ouml;rt
	 * @param name
	 *            Der Name des Terms
	 * @param zugehoerigkeit
	 *            Die Zugeh&ouml;rigkeit des Terms
	 */
	Term(final FuzzyVariable fuzzyVariable, final String name,
			final Float zugehoerigkeit) {
		super(zugehoerigkeit);
		this.name = name;
		this.fuzzyVariable = fuzzyVariable;
	}

	/**
	 * Zwei Terme sind "gleich", wenn sie den selben Namen besitzen. Dieses
	 * Verhalten ist f&uuml;r das Hashing notwendig.
	 *
	 * {@inheritDoc}
	 *
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(final Object obj) {
		if (obj == this) {
			return true;
		} else if (obj instanceof Term) {
			final Term term = (Term) obj;
			return name.equals(term.name);
		}

		return false;
	}

	@Override
	public int hashCode() {
		final int hash1 = name != null ? name.hashCode() : 0;
		final int hash2 = fuzzyVariable != null ? fuzzyVariable.hashCode() : 0;
		return hash1 ^ hash2;
	}

	/**
	 * Sucht das Fuzzy-Set, was zu diesem Term geh&ouml;rt.
	 *
	 * @return Das gesuchte Fuzzy-Set oder {@code null}, wenn keines existiert
	 */
	public FuzzySet getFuzzySet() {
		if (fuzzyVariable.getLinguistischeVariable() == null) {
			return null;
		}

		return fuzzyVariable.getLinguistischeVariable().getFuzzySet(name);
	}

	/**
	 * Gibt die Fuzzy-Variable zur&uuml;ck, zu der dieser Term geh&ouml;rt.
	 *
	 * @return Fuzzy-Variable
	 */
	public FuzzyVariable getFuzzyVariable() {
		return fuzzyVariable;
	}

	/**
	 * Gibt den Namen des Terms zur&uuml;ck.
	 *
	 * @return Termname
	 */
	public String getName() {
		return name;
	}

	/**
	 * Gibt den Term als Tupel "Name: Zugeh&ouml;rigkeit" zur&uuml;ck.
	 *
	 * {@inheritDoc}
	 *
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return name + ": " + getZugehoerigkeit(); 
	}

}
